@@ -39,28 +39,4 @@ sub refresh {
     $self->configure_modperl;
 }
 
-# generate t/TEST script (or a different filename) which will drive
-# Apache::TestRunPerl
-sub generate_script {
-    my ($class, $file) = @_;
-
-    $file ||= catfile 't', 'TEST';
-
-    my $content = <<'EOM';
-use strict;
-use warnings FATAL => 'all';
-
-use FindBin;
-use lib "$FindBin::Bin/../Apache-Test/lib";
-
-use Apache::TestRunPerl ();
-
-Apache::TestRunPerl->new->run(@ARGV);
-EOM
-
-    Apache::Test::config()->write_perlscript($file, $content);
-
-}
-
-
 1;
