@@ -5,6 +5,11 @@ use warnings FATAL => 'all';
 
 use Apache::Test;
 
+use APR::Table ();
+use Apache::RequestRec ();
+
+use Apache::Const -compile => 'OK';
+
 sub first {
     my $r = shift;
 
@@ -53,5 +58,6 @@ PerlInitHandler TestHooks::init::second
 <Base>
     PerlInitHandler TestHooks::init::first
 </Base>
+PerlResponseHandler TestHooks::init
 PerlResponseHandler TestHooks::init::response
 SetHandler modperl
