@@ -1,13 +1,22 @@
+#ifdef WIN32
+/* win32 not happy with &PL_sv_no */
+#   define SVNO  newSViv(0)
+#   define SVYES newSViv(1)
+#else
+#   define SVNO  &PL_sv_no
+#   define SVYES &PL_sv_yes
+#endif
+
 #define mpxs_Apache__RequestRec_TIEHANDLE(stashsv, sv) \
 modperl_newSVsv_obj(aTHX_ stashsv, sv)
 
 #define mpxs_Apache__RequestRec_PRINT  mpxs_Apache__RequestRec_print
 #define mpxs_Apache__RequestRec_PRINTF mpxs_ap_rprintf
 #define mpxs_Apache__RequestRec_BINMODE(r) \
-    r ? &PL_sv_yes : &PL_sv_no /* noop */
+    r ? SVYES : SVNO /* noop */
 
 #define mpxs_Apache__RequestRec_UNTIE(r, refcnt) \
-    (r && refcnt) ? &PL_sv_yes : &PL_sv_no /* noop */
+    (r && refcnt) ? SVYES : SVNO /* noop */
 
 #define mpxs_output_flush(r, rcfg) \
     /* if ($|) */ \
@@ -97,11 +106,11 @@ apr_size_t mpxs_ap_rprintf(pTHX_ I32 items, SV **MARK, SV **SP)
     mpxs_Apache__RequestRec_write(aTHX_ r, buffer, bufsiz, offset)
 
 static MP_INLINE
-apr_ssize_t mpxs_Apache__RequestRec_write(pTHX_ request_rec *r,
-                                          SV *buffer, apr_ssize_t bufsiz,
-                                          int offset)
+apr_size_t mpxs_Apache__RequestRec_write(pTHX_ request_rec *r,
+                                         SV *buffer, apr_size_t bufsiz,
+                                         int offset)
 {
-    apr_ssize_t wlen = bufsiz;
+    apr_size_t wlen = bufsiz;
     const char *buf;
     STRLEN svlen;
     MP_dRCFG;
